//+------------------------------------------------------------------+
//|                                       Heiken Ashi Martingale.mq4 |
//|                                    Copyright  2007, Ryan Klefas |
//|                                        http://www.metaquotes.net |
//+------------------------------------------------------------------+
#property copyright "Copyright  2007, Ryan Klefas (Base 1.7)"
#property link      "rklefas@inbox.com"



extern string    basic="==== Basic Settings ====================";
extern double    TakeProfit=55;
extern double    Stoploss=50;
extern double    Lots=0.1;

extern string    time="==== Time Settings ====================";
extern int       StartHour=0;          // Entry hour according to GMT
extern int       CloseOutHour=23;      // Hour to close existing orders according to GMT

extern string    lot="==== Lot Sizing ====================";
extern bool      Martingale=true;
extern int       MartingaleMultiplier=2; 

extern string    om="==== Order Management ====================";
extern int       breakEvenAtProfit=0;
extern int       breakEvenShift=0;
extern int       TrailingStop=0;
extern bool      OnlyTrailAfterProfit=false;
extern int       SteppingStop=0;
extern bool      OnlyStepAfterProfit=false;

extern string    account="==== Account Settings ====================";
extern int       MinimumEquity=0;
extern int       MinimumBalance=0;

extern string    activity="==== EA Activity Settings ====================";
extern bool      DisableClosingOrders=false;
extern bool      DisableNewOrders=false;
extern int       MaximumOrders=10;

extern string    id="==== Identity Settings ====================";
extern bool      IdentitySensitivity_Low=false;
extern int       magicNum=14534711;
extern string    shortName="HA Martingale";


//+------------------------------------------------------------------+
//| expert initialization function                                   |
//+------------------------------------------------------------------+
int init()
  {

   return(0);
  }
//+------------------------------------------------------------------+
//| expert deinitialization function                                 |
//+------------------------------------------------------------------+
int deinit()
  {

   return(0);
  }
//+------------------------------------------------------------------+
//| expert start function                                            |
//+------------------------------------------------------------------+
int start()
  {
   
   int allOrders=0, buyOrders=0, sellOrders=0, buyPending=0, sellPending=0;
   checkOrderStatus(allOrders,buyOrders,sellOrders,buyPending,sellPending);
   advancedStopManager(buyOrders,sellOrders);

//+------------------------------------------------------------------+
//| variable and indicator declaration                               |
//+------------------------------------------------------------------+



double haOpen = iCustom(NULL,0,"Heiken_Ashi_Smoothed",2,6,3,2,2,1) ;
double haClose = iCustom(NULL,0,"Heiken_Ashi_Smoothed",2,6,3,2,3,1) ;






//+------------------------------------------------------------------+
//| close existing orders under apprpriate conditions                |
//+------------------------------------------------------------------+
   if (DisableClosingOrders==false && allOrders>0)  {
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+


if(Hour() == CloseOutHour) 
{
   if (buyOrders>0)
      closeBuyOrder();
   if (sellOrders>0)
      closeSellOrder();
}
      
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
      }
//+------------------------------------------------------------------+
//| create new orders                                                |
//+------------------------------------------------------------------+
   if (newOrderPrevention(allOrders)==false)  {
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+




if(Hour() == StartHour) 
{
   if (allOrders==0)
   {
      if(haOpen < haClose)
         sendBuyOrder();
      if(haOpen > haClose)
         sendSellOrder();
   }
}




//+------------------------------------------------------------------+
//+------------------------------------------------------------------+

      }
   return(0);
  }
  
//+------------------------------------------------------------------+
//| middle-man modules                                               |
//+------------------------------------------------------------------+

void sendBuyOrder() // Standard Version 1.7
{ versatileOrderTaker(1, Ask); }
   
//+------------------------------------------------------------------+

void sendSellOrder() // Standard Version 1.7
{ versatileOrderTaker(2, Bid); }

//+------------------------------------------------------------------+

void closeSellOrder() // Standard Version 1.7
{ versatileOrderCloser(2); }

//+------------------------------------------------------------------+

void closeBuyOrder() // Standard Version 1.7
{ versatileOrderCloser(1); }

//+------------------------------------------------------------------+





//+------------------------------------------------------------------+
//| modules and functions                                            |
//+------------------------------------------------------------------+

void trailingStopManager() // Standard Version 1.7
{
   
   int cnt, total = OrdersTotal();

   for(cnt=0;cnt<total;cnt++)
   {
      OrderSelect(cnt, SELECT_BY_POS, MODE_TRADES);

      if (TrailingStop>0 && orderBelongsToMe())
      {
         if (OrderType()==OP_BUY)
         {
            if ((Bid-OrderOpenPrice()>Point*TrailingStop) || OnlyTrailAfterProfit==false)
            {
               if (OrderStopLoss()<Bid-Point*TrailingStop)
                  OrderModify(OrderTicket(),OrderOpenPrice(),Bid-Point*TrailingStop,OrderTakeProfit(),0,Green); 
            }
         }
         else if (OrderType()==OP_SELL)
         {
            if ((OrderOpenPrice()-Ask>Point*TrailingStop) || OnlyTrailAfterProfit==false)
            {
               if ((OrderStopLoss()>(Ask+Point*TrailingStop)) || (OrderStopLoss()==0))
                  OrderModify(OrderTicket(),OrderOpenPrice(),Ask+Point*TrailingStop,OrderTakeProfit(),0,Red); 
            }
         }
      }  
   }




}

//+------------------------------------------------------------------+

void breakEvenManager() // Standard Version 1.7
{

   for(int cnt=0;cnt<OrdersTotal();cnt++)
   {
   OrderSelect(cnt, SELECT_BY_POS, MODE_TRADES);

      if (breakEvenAtProfit>0 && orderBelongsToMe())
      {
         if (OrderType()==OP_BUY)
         {
            if (Bid-OrderOpenPrice()>=Point*breakEvenAtProfit)
            {
               if (OrderStopLoss()!=OrderOpenPrice() + breakEvenShift*Point)
                  OrderModify(OrderTicket(),OrderOpenPrice(),OrderOpenPrice()+ breakEvenShift*Point,OrderTakeProfit(),0,Green); 
            }
         }
         else if (OrderType()==OP_SELL)
         {
            if (OrderOpenPrice()-Ask>=Point*breakEvenAtProfit)
            {
               if (OrderStopLoss()!=OrderOpenPrice() - breakEvenShift*Point)
                  OrderModify(OrderTicket(),OrderOpenPrice(),OrderOpenPrice()- breakEvenShift*Point,OrderTakeProfit(),0,Red); 
            }
         }
      }
   }


}
   
//+------------------------------------------------------------------+

 // Standard Version 1.7
void checkOrderStatus(int& allOrders,int& buyOrders,int& sellOrders,int& buyPending, int& sellPending)
{
   for(int cnt=OrdersTotal();cnt>=0;cnt--)
   {
      OrderSelect(cnt, SELECT_BY_POS, MODE_TRADES);
      if ( orderBelongsToMe() )
      {  
         if (OrderType()==OP_BUY)
            {buyOrders++; allOrders++;}
         else if (OrderType()==OP_SELL)
            {sellOrders++; allOrders++;}
         else if (OrderType()==OP_BUYSTOP || OrderType()==OP_BUYLIMIT)
            {buyPending++; allOrders++;}
         else if (OrderType()==OP_SELLSTOP || OrderType()==OP_SELLLIMIT)
            {sellPending++; allOrders++;}
      }
   }
}

//+------------------------------------------------------------------+

void advancedStopManager(int buyOrders, int sellOrders) // Standard Version 1.7
{
    
   if (buyOrders>=1 || sellOrders>=1)
   {
      if (breakEvenAtProfit>0)
         breakEvenManager();
      if (TrailingStop>0)
         trailingStopManager();
      if (SteppingStop>0)
         steppingStopManager();
    }
}

//+------------------------------------------------------------------+

void steppingStopManager() // Standard Version 1.7
{

   int cnt, total = OrdersTotal();
   double stepStopValue;
   
   for(cnt=0;cnt<total;cnt++)
   {
      OrderSelect(cnt, SELECT_BY_POS, MODE_TRADES);

      if (SteppingStop>0 && orderBelongsToMe())
      {
         if (OrderType()==OP_BUY)
         {
            if ( (Bid-OrderOpenPrice()>Point*SteppingStop) || OnlyStepAfterProfit==false)
            {
               if (OrderStopLoss()<Bid-Point*SteppingStop)
               {
                  stepStopValue=OrderStopLoss();
                  while (stepStopValue<Bid-Point*SteppingStop)
                  {
                     stepStopValue = stepStopValue+SteppingStop*Point;
                  }
                  OrderModify(OrderTicket(),OrderOpenPrice(),stepStopValue,OrderTakeProfit(),0,Green); 
               }     
            }
         }
         else if (OrderType()==OP_SELL)
         {

            if ((OrderOpenPrice()-Ask>Point*SteppingStop) || OnlyStepAfterProfit==false)
            {

               if ((OrderStopLoss()>(Ask+Point*SteppingStop)) || (OrderStopLoss()==0))
               {
                  if (OrderStopLoss()>0)
                     stepStopValue=OrderStopLoss();
                  else
                     stepStopValue = Ask*2;  
                  while (stepStopValue>Ask+Point*SteppingStop)
                  {
                     stepStopValue = stepStopValue-SteppingStop*Point;
                  }
                  OrderModify(OrderTicket(),OrderOpenPrice(),OrderStopLoss()-Point*SteppingStop,OrderTakeProfit(),0,Red); 
               }
            }
         }
      }
   }



}

//+------------------------------------------------------------------+

bool orderBelongsToMe() // Standard Version 1.7
{
   if (OrderSymbol()==Symbol() && OrderMagicNumber()==simpleMagicGenerator() )
       return (true);
   else
      return (false);
}

//+------------------------------------------------------------------+

int simpleMagicGenerator() // Standard Version 1.7
{

   int truMagic;
   
   if (IdentitySensitivity_Low)
      truMagic = magicNum;      
   else
      truMagic = magicNum + (Period()*17 + 203);

   return (truMagic);

}

//+------------------------------------------------------------------+

string commentString() // Standard Version 1.7
{
   string tfString, fullComment;
   
   switch (Period())  
   {   
      case 1:      tfString = "1 Min";    break;
      case 5:      tfString = "5 Min";    break;
      case 15:     tfString = "15 Min";   break;  
      case 30:     tfString = "30 Min";   break;  
      case 60:     tfString = "1 Hour";   break;  
      case 240:    tfString = "4 Hour";   break;  
      case 1440:   tfString = "Daily";    break;  
      case 10080:  tfString = "Weekly";   break;  
      case 40320:  tfString = "Monthly";  break;  
      default:     tfString = Period();   
   }

   fullComment=StringConcatenate(shortName, ": ", tfString);
   return (fullComment);
}

//+------------------------------------------------------------------+

double lotMaker() // Standard Version 1.71
{

// Log history for easy access by Lot Calculators

   int myHistoryTotal=0;

   if (Martingale)
   {
      int cnt, total = OrdersHistoryTotal();
   
      if (total>0)
      {
   
         int historyPosition[1];
         int historyWinLoss[1];
         double historyLotSize[1];
         


         for(cnt=0;cnt<total;cnt++)
         {
            OrderSelect(cnt, SELECT_BY_POS, MODE_HISTORY);
            if ( orderBelongsToMe() )
               {myHistoryTotal++;}
         }
         
         if (myHistoryTotal>1)
         {
            ArrayResize(historyPosition, myHistoryTotal);
            ArrayResize(historyLotSize, myHistoryTotal);
            ArrayResize(historyWinLoss, myHistoryTotal);
         }
         
         
         int positionCounter = myHistoryTotal-1;

         if (ArrayInitialize(historyPosition,0)==true && 
             ArrayInitialize(historyLotSize,0.0)==true &&
             ArrayInitialize(historyWinLoss,8)==true)
            Print("All arrays initialized");
         else  
            Print("Array initialization error: " , GetLastError());


if (positionCounter<0)
   positionCounter=0;

         for(cnt=0;cnt<total;cnt++)
         {
            OrderSelect(cnt, SELECT_BY_POS, MODE_HISTORY);

            if ( orderBelongsToMe() )
            {
               historyPosition[positionCounter] = cnt;
               historyLotSize[positionCounter]=OrderLots();

               if (OrderClosePrice()==OrderTakeProfit())
                  {
                  historyWinLoss[positionCounter]=1;
                  }
               else if (OrderClosePrice()==OrderStopLoss())
                  {
                  historyWinLoss[positionCounter]=-1;
                  }
               else 
                  {
                  historyWinLoss[positionCounter]=0; 
                  }
                  
               positionCounter--;
            }
         }
      }
   }
   
// Call appropriate Lot Calculator

   double finalLot=Lots;
  
   if (myHistoryTotal>0)
   {
      if (Martingale)
         finalLot = MartingaleLots(historyPosition, historyWinLoss, historyLotSize);
   }   
   
   if (finalLot>100)
      finalLot=100;
   
   return (finalLot);

}


//+------------------------------------------------------------------+

// Standard Version 1.7
double MartingaleLots(int historyPosition[], int historyWinLoss[], double historyLotSize[]) 
{

   int xx=0;
   OrderSelect(historyPosition[xx], SELECT_BY_POS, MODE_HISTORY);

   if (historyWinLoss[xx]==-1)
      return (OrderLots()*MartingaleMultiplier);
   else if (historyWinLoss[xx]==0)
      return (OrderLots());
   else 
      { return (Lots);}

}

//+------------------------------------------------------------------+

bool newOrderPrevention(int allOrders) // Standard Version 1.7
{

   if (DisableNewOrders)
      { Comment("New Orders Disabled:  User option"); return (true); }
   else if (AccountEquity()<MinimumEquity)
      { Comment("New Orders Disabled:  Equity too low"); return (true);}
   else if (AccountBalance()<MinimumBalance)
      { Comment("New Orders Disabled:  Balance too low"); return (true);}
   else if (allOrders>=MaximumOrders)
      { Comment("New Orders Disabled:  Too many existing orders"); return (true);}
   else
      return (false);

}

//+------------------------------------------------------------------+

void versatileOrderTaker(int simpleType, double entryPrice) // Standard Version 1.7
{

   // Variable calculations

   double Stopvalue=0, TPvalue=0;
   int ticket, oType, slippage=3;
   string typeName;


   switch (simpleType)
   {
      case 4: 
      if (entryPrice<Bid)  { oType=OP_SELLSTOP;  typeName="SELLSTOP"; }
      if (entryPrice>Bid)  { oType=OP_SELLLIMIT; typeName="SELLLIMIT"; }
      if (Stoploss>0)      {Stopvalue = entryPrice+Stoploss*Point;}
      if (TakeProfit>0)    {TPvalue = entryPrice-TakeProfit*Point;}
      break;
         
      case 3:
      if (entryPrice>Ask)  { oType=OP_BUYSTOP;  typeName="BUYSTOP"; }
      if (entryPrice<Ask)  { oType=OP_BUYLIMIT; typeName="BUYLIMIT"; }
      if (Stoploss>0)      {Stopvalue = entryPrice-Stoploss*Point;}
      if (TakeProfit>0)    {TPvalue = entryPrice+TakeProfit*Point;}
      break;

      case 2:
      oType=OP_SELL; typeName="SELL"; 
      if (Stoploss>0)      {Stopvalue = entryPrice+Stoploss*Point;}
      if (TakeProfit>0)    {TPvalue = entryPrice-TakeProfit*Point;}
      break;

      case 1:
      oType=OP_BUY; typeName="BUY"; 
      if (Stoploss>0)      {Stopvalue = entryPrice-Stoploss*Point;}
      if (TakeProfit>0)    {TPvalue = entryPrice+TakeProfit*Point;}
      break;
      
      default:
      Print ("versatileOrderTaker has been passed an invalid SimpleType parameter: " + simpleType);
   }

   // Send Order
   
   ticket=OrderSend(Symbol(),oType,lotMaker(),entryPrice,slippage,Stopvalue,TPvalue,commentString(),simpleMagicGenerator(),0,Green);
   if (ticket>0)
     {if (OrderSelect(ticket,SELECT_BY_TICKET,MODE_TRADES)) Print(shortName + " " + typeName + " order at ",OrderOpenPrice()); }
   else 
      Print("Error opening " + typeName + " order: ",GetLastError());
}

//+------------------------------------------------------------------+

void versatileOrderCloser(int simpleType) // Standard Version 1.7
{
   int cnt, total = OrdersTotal();

   for(cnt=0;cnt<total;cnt++)
   {
      OrderSelect(cnt, SELECT_BY_POS, MODE_TRADES);
      
      if ( orderBelongsToMe() )
      {
         switch (simpleType)
         {
            case 4:
            case 3:
               if (OrderType()>OP_SELL)   OrderDelete(OrderTicket());
            break;
         
            case 2:
               if (OrderType()==OP_SELL)  OrderClose(OrderTicket(),OrderLots(),Ask,3,CLR_NONE); 
            break;
         
            case 1:
               if (OrderType()==OP_BUY)   OrderClose(OrderTicket(),OrderLots(),Bid,3,CLR_NONE); 
            break;
         
            default:
            Print ("versatileOrderCloser has been passed an invalid SimpleType parameter: " + simpleType);
         }
      }
   }
}

//+------------------------------------------------------------------+


//+------------------------------------------------------------------+


//+------------------------------------------------------------------+


//+------------------------------------------------------------------+


//+------------------------------------------------------------------+

